@@ -585,6 +585,8 @@ ags_core_audio_devin_init(AgsCoreAudioDevin *core_audio_devin)
 {
   AgsConfig *config;
 
+  GList *start_note_256th_attack;
+
   gchar *str;
   gchar *segmentation;
 
@@ -675,6 +677,20 @@ ags_core_audio_devin_init(AgsCoreAudioDevin *core_audio_devin)
   core_audio_devin->attack = (guint *) g_malloc((int) 2 * AGS_SOUNDCARD_DEFAULT_PERIOD *
 					       sizeof(guint));
 
+  start_note_256th_attack = NULL;
+
+  for(i = 0; i < 16; i++){
+    guint *note_256th_attack;
+    
+    note_256th_attack = (guint *) malloc(2 * (int) AGS_SOUNDCARD_DEFAULT_PERIOD *
+					 sizeof(guint));
+    
+    start_note_256th_attack = g_list_prepend(start_note_256th_attack,
+					     note_256th_attack);
+  }
+
+  core_audio_devin->note_256th_attack = start_note_256th_attack;
+
   ags_core_audio_devin_adjust_delay_and_attack(core_audio_devin);
   
   /* counters */
