@@ -597,6 +597,8 @@ ags_jack_devout_init(AgsJackDevout *jack_devout)
 {
   AgsConfig *config;
 
+  GList *start_note_256th_attack;
+  
   gchar *str;
   gchar *segmentation;
 
@@ -683,6 +685,20 @@ ags_jack_devout_init(AgsJackDevout *jack_devout)
   jack_devout->attack = (guint *) g_malloc((int) 2 * AGS_SOUNDCARD_DEFAULT_PERIOD *
 					   sizeof(guint));
 
+  start_note_256th_attack = NULL;
+
+  for(i = 0; i < 16; i++){
+    guint *note_256th_attack;
+    
+    note_256th_attack = (guint *) malloc(2 * (int) AGS_SOUNDCARD_DEFAULT_PERIOD *
+					 sizeof(guint));
+    
+    start_note_256th_attack = g_list_prepend(start_note_256th_attack,
+					     note_256th_attack);
+  }
+
+  jack_devout->note_256th_attack = start_note_256th_attack;
+
   ags_jack_devout_adjust_delay_and_attack(jack_devout);
   
   /* counters */
